package io.gitlab.jfronny.inceptum.common.model.maven;

import java.util.*;

public class DependencyNode {
    private final String name;
    private final Set<DependencyNode> dependencies;

    public DependencyNode(Pom pom, Set<DependencyNode> dependencies) {
        Objects.requireNonNull(pom);
        this.name = pom.groupId + ":" + pom.artifactId + ":" + pom.version;
        this.dependencies = Objects.requireNonNull(dependencies);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        DependencyNode that = (DependencyNode) o;
        return name.equals(that.name) && dependencies.equals(that.dependencies);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, dependencies);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        generateTree(sb, "", "");
        return sb.toString();
    }

    private void generateTree(StringBuilder sb, String prefix, String childrenPrefix) {
        sb.append(prefix).append(name).append('\n');
        for (Iterator<DependencyNode> it = dependencies.iterator(); it.hasNext();) {
            DependencyNode next = it.next();
            if (it.hasNext()) {
                next.generateTree(sb, childrenPrefix + "├── ", childrenPrefix + "│   ");
            } else {
                next.generateTree(sb, childrenPrefix + "└── ", childrenPrefix + "    ");
            }
        }
    }
}
