package io.gitlab.jfronny.inceptum.launcher.model.mojang;

import io.gitlab.jfronny.gson.compile.annotations.GSerializable;

import java.util.*;

@SuppressWarnings("MethodDoesntCallSuperMethod")
@GSerializable
public class VersionInfo extends VersionsListInfo implements Cloneable {
    public Arguments arguments;
    public AssetIndex assetIndex;
    public String assets;
    public Downloads downloads;
    public JavaVersion javaVersion;
    public List<Library> libraries;
    public String mainClass;
    public String minecraftArguments;
    public Integer minimumLauncherVersion;

    @Override
    public VersionInfo clone() {
        VersionInfo clone = new VersionInfo();
        clone.copyFrom(this);
        if (arguments != null) clone.arguments = arguments.clone();
        if (assetIndex != null) clone.assetIndex = assetIndex.clone();
        clone.assets = assets;
        if (downloads != null) clone.downloads = downloads.clone();
        if (javaVersion != null) clone.javaVersion = javaVersion.clone();
        clone.libraries = new LinkedList<>();
        if (libraries != null) libraries.forEach(library -> clone.libraries.add(library.clone()));
        clone.mainClass = mainClass;
        clone.minecraftArguments = minecraftArguments;
        clone.minimumLauncherVersion = minimumLauncherVersion;
        return clone;
    }

    @GSerializable
    public static class Arguments implements Cloneable {
        public List<MinecraftArgument> game;
        public List<MinecraftArgument> jvm;

        @Override
        public Arguments clone() {
            Arguments clone = new Arguments();
            clone.game = new LinkedList<>();
            if (game != null) game.forEach(argument -> clone.game.add(argument.clone()));
            clone.jvm = new LinkedList<>();
            if (jvm != null) jvm.forEach(argument -> clone.jvm.add(argument.clone()));
            return clone;
        }
    }

    @GSerializable
    public static class AssetIndex extends MojangFileDownload {
        public String id;
        public int totalSize;

        @Override
        public AssetIndex clone() {
            AssetIndex clone = new AssetIndex();
            clone.copyFrom(this);
            clone.id = id;
            clone.totalSize = totalSize;
            return clone;
        }
    }

    @GSerializable
    public static class Downloads implements Cloneable {
        public MojangFileDownload client;
        public MojangFileDownload client_mappings;
        public MojangFileDownload server;
        public MojangFileDownload server_mappings;

        @Override
        public Downloads clone() {
            Downloads clone = new Downloads();
            clone.client = client.clone();
            if (client_mappings != null) clone.client_mappings = client_mappings.clone();
            clone.server = server.clone();
            if (server_mappings != null) clone.server_mappings = server_mappings.clone();
            return clone;
        }
    }

    @GSerializable
    public static class JavaVersion implements Cloneable {
        public String component;
        public int majorVersion;

        @Override
        public JavaVersion clone() {
            JavaVersion clone = new JavaVersion();
            clone.component = component;
            clone.majorVersion = majorVersion;
            return clone;
        }
    }

    @GSerializable
    public static class Library implements Cloneable {
        public Downloads downloads;
        public String name;
        public Map<String, String> natives;
        public Rules rules;

        @Override
        public Library clone() {
            Library clone = new Library();
            if (downloads != null) clone.downloads = downloads.clone();
            clone.name = name;
            clone.natives = new LinkedHashMap<>();
            if (natives != null) clone.natives.putAll(natives);
            if (rules != null) clone.rules = rules.clone();
            return clone;
        }

        @GSerializable
        public static class Downloads implements Cloneable {
            public Artifact artifact;
            public Map<String, Artifact> classifiers;

            @Override
            public Downloads clone() {
                Downloads clone = new Downloads();
                if (artifact != null) clone.artifact = artifact.clone();
                clone.classifiers = new LinkedHashMap<>();
                if (classifiers != null) classifiers.forEach((key, value) -> clone.classifiers.put(key, value.clone()));
                return clone;
            }

            @GSerializable
            public static class Artifact extends MojangFileDownload implements Cloneable {
                public String path;

                @Override
                public Artifact clone() {
                    Artifact clone = new Artifact();
                    clone.copyFrom(this);
                    clone.path = path;
                    return clone;
                }
            }
        }
    }
}
